///////    stringSearch.C    ///////
#include        <iostream>
#include        <string>
#include        <fstream>
#include        <stdlib.h>

int stringMatch(std::string& str, std::string& line)     // (1)
{   if ( str.empty() ) return -1;                         // (2)
    int j, k;
    int sl= str.size(), ll = line.size();  // string length (3)
    for (int i = 0 ; i < ll ; i++)
    {   if ( line[i] != str[0] )   // first chars different (4)
              continue;            // skip rest of loop body
        // compare remaining characters
        // until first mismatch or end of string
        for ( j = i + 1, k = 1;
              line[j]==str[k] && k < sl ;  // (5)
              j++, k++
            ) { }
        if ( k == sl ) return i;   // successful match
        else 
            if (j == ll) return -1; // no match possible any more
    }
    return -1;                      // failed to match
}

void match(std::string& str, std::istream& in, std::ostream& out)
{   if ( str.empty() )        // str is empty
    {   std::cerr << "match: empty match string"
                  << std::endl;
        exit(1);
    }
    std::string line;
    while ( std::getline(in, line, '\n') )
    {   if ( stringMatch(str, line) > -1 )
            out << line << std::endl;
    }
    out.flush();
}

int main (int argc,  char* argv[])
{   if ( argc < 2 || argc > 4 )   // error
    {   std::cerr << "Usage:" << argv[0] 
                  << " str [infile [outfile]]" << std::endl;
        return 1;
    }
    std::string str(argv[1]);
    if ( argc == 2 )      // use standard I/O
        match(str, std::cin, std::cout);
    else                  // use file
    {   ifstream infile(argv[2]);
        if ( infile.fail() )
        {   std::cerr << argv[0] << ":can't open input file "
                 << argv[2] << std::endl;
            return 1;
        }
        if ( argc == 3 ) 
            match(str, infile, std::cout);
        else // argc == 4
        {   ofstream ofile(argv[3]);
            if ( ofile.fail() )
            {   std::cerr << argv[0] << ":can't open output file " 
                     << argv[3] << std::endl;
                exit(1);
            }
            match(str, infile, ofile);
        }
    }
    return 0;
}
