///////    Matrix.C    ///////
#include <assert.h>
#include <math.h>
#include "Matrix.h"

Matrix::Matrix(int r, int c)
{   setUp(r,c);   }

void Matrix::setUp(int r, int c)
{   if ( r > 0 && c > 0 )
    {   nr = r; nc = c;
        mat = new double[r * c];
    } 
    else 
    {   mat = NULL; nr=nc=0;   }
}


double Matrix::getElement(int i, int j) const
{   if ( 0<=i && i<nr && 0<=j && j<nc )
         return mat[i*nc+j];
    else return HUGE_VAL;   // illegal value
}

void Matrix::setElement(int i, int j, double e)
{   if ( 0<=i && i<nr && 0<=j && j<nc )
         mat[i*nc+j] = e;
}

Matrix::Matrix(double* m, int r, int c) 
{   setUp(r,c);
    if ( mat != NULL )  // copy data
         for (int i=0; i < r*c; i++)
                  mat[i] = m[i];
}

void Matrix::times(const Matrix& b, Matrix& ans) const
{   assert( nc == b.nr );   // compatible dimensions
    for (int i=0 ; i < nr ; i++)
    {  for (int j=0 ; j < b.nc; j++)
           ans.setElement(i, j, 
                rowTimesCol(i, b.mat, j, b.nc));
    }
}

void Matrix::display() const
{   for (int i = 0; i < nr; i++)
    {  std::cout << "( ";
       for (int j = 0; j < nc-1; j++)
           std::cout << mat[i*nc+j] << "  ";
       std::cout << mat[i*nc+nc-1];
       std::cout << " )" << std::endl;
    }
}

double Matrix::rowTimesCol(int i, double* b, int j, int bc) const
{   double sum=0.0;
    for (int k=0; k < nc; k++)
        sum += mat[i*nc+k] * b[k*bc+j];
    return sum;
}
