///////    ArbList.C    ///////
#include <iostream>
#include "ArbList.h"
using std::cout;

class ArbCell
{ friend class ArbList;
  private:
    Any item;
    ArbCell* next;
    ArbCell(Any c = NULL, ArbCell* ptr = NULL)
    :  item(c), next(ptr) { }    // constructor
};

ArbCell* ArbList::next(ArbCell* p)
{   return p->next;  }

ArbList::ArbList(Any c, EQ_FN eq, DISP_FN d)
: head (new ArbCell(c,NULL)),
equal(eq), dispfn(d) { }

ArbList::~ArbList() {   free();  }

void ArbList::free() // private method
{     ArbCell* n, *p = head;
      while (p)
      {   n = p->next;
          delete p;
          p=n;
      }
}

int ArbList::shorten(int n)
{     while (n-- && head)
      {   ArbCell* tmp=head;
          head = head->next;
          delete(tmp);
      }
      return(- ++n);
}

int ArbList::remove(Any c)
{     ArbCell *tmp, *p = head;
      int count = 0;
      if ( p == NULL ) return count;
      while (p->next)      // treat all but head cell
      {  if ( equal((p->next)->item, c) ) // call equal
         {   count++;
             tmp = p->next;
             p->next = tmp->next;
             delete(tmp); // free up store
         }
         else
                p = p->next;
      }
      if( equal(head->item, c) ) // treat head cell
      {  tmp = head;
         head = head->next;
         delete(tmp);
         count++;
      }
      return count;        // number of items removed
}

bool ArbList::putOn(Any c)
{     ArbCell* tmp = new ArbCell(c,head);
      if ( tmp )
      {  head = tmp;
         return true;
      }
      else return false; // failed
}

// insert after entry e
bool ArbList::insert(Any c, ArbCell* e)
{     if ( e == NULL ) return putOn(c);
      ArbCell* tmp = new ArbCell(c,e->next);
      if ( tmp )
      {  if (e) e->next = tmp;
         else head = tmp;
         return true;
      }
      else return false; // failed
}

ArbCell* ArbList::last()
{     ArbCell* p = head;
      while( p && p->next ) p =  p->next; 
      return p;
}

Any ArbList::content(ArbCell* p)         // content of list item
{   return p->item;   }

ArbCell* ArbList::find(Any c)
{   for( ArbCell* p = head; p ; p = p->next )
       if( equal(p->item, c) )
           return p;
    return NULL;  // c not on list
}

bool ArbList::substitute(Any r, Any s)
{   ArbCell* p = find(s);
    if ( p == NULL ) return false;  // s not on list
    p->item = r;
    return true;
}

// display from p to end
void ArbList::display(ArbCell* p) const
{   cout << "(";
    while ( p )
    {    dispfn(p->item);
         if ( p = p->next ) cout << " ";
    }
    cout << ")";
}
