///////    Matrix.C    ///////
#include <iostream>
#include <strstream>
#include <string>
#include "Matrix.h"
using std::ostrstream; using std::string;
using std::cout; using std::endl;

string int2String(int i)
{   int SIZE = 64;
    char buf[SIZE];
    ostrstream mystr(buf, SIZE);
    mystr << i << '\0';
    return string(buf);  
}

Matrix::Matrix(int r, int c)
{   setUp(r,c);   }

void Matrix::setUp(int r, int c)
{   if ( r > 0 && c > 0 )
    {   nr = r; nc = c;
        mat = new double[r * c];
    } 
    else 
    {   mat = NULL; nr=nc=0;   }
}



double Matrix::getElement(int i, int j) const
	 throw(InvalidIndex)
{   if ( 0>i || i>=nr || 0>j || j>=nc )
	 throw(InvalidIndex(string("getElement(")
	       + int2String(i) + "," + int2String(j) + ")" 
	       + " of ("  + int2String(nr) + " x " 
	       + int2String(nc) + ")" + " matrix." ) );
    return mat[i*nc+j];
}

void Matrix::setElement(int i, int j, double e)
	 throw(InvalidIndex)
{   if ( 0>i || i>=nr || 0>j || j>=nc )
	 throw(InvalidIndex(string("setElement(")
	       + int2String(i) + "," + int2String(j) + ")" 
	       + " of ("  + int2String(nr) + " x " 
	       + int2String(nc) + ")" + " matrix." ) );
    mat[i*nc+j] = e;
}

Matrix::Matrix(double* m, int r, int c) 
{   setUp(r,c);
    if ( mat != NULL )  // copy data
         for (int i=0; i < r*c; i++)
	          mat[i] = m[i];
}

Matrix& Matrix::times(const Matrix& b) const
        throw(IncompatibleDimension)
{   if ( nc != b.nr )   // incompatible dimensions
        throw(IncompatibleDimension(int2String(nr)
	    + " x " + int2String(nc) + " times "
	    + int2String(b.nr) + " x " + int2String(b.nc)));
    Matrix *ans = new Matrix(nr, b.nc);
    for (int i=0 ; i < nr ; i++)
    {  for (int j=0 ; j < b.nc; j++)
           ans->setElement(i, j, 
                rowTimesCol(i, b.mat, j, b.nc));
    }
    return *ans;
}

void Matrix::display() const
{   for (int i = 0; i < nr; i++)
    {  std::cout << "( ";
       for (int j = 0; j < nc-1; j++)
           std::cout << mat[i*nc+j] << "  ";
       std::cout << mat[i*nc+nc-1];
       std::cout << " )" << std::endl;
    }
}

double Matrix::rowTimesCol(int i, double* b, int j, int bc) const
{   double sum=0.0;
    for (int k=0; k < nc; k++)
        sum += mat[i*nc+k] * b[k*bc+j];
    return sum;
}
