///////    Fraction.C    ///////
#include "Fraction.h"

inline int ABS(int x) { return(x < 0 ? -x : x); } 

unsigned int gcd(int a, int b)
{       int t;
        a = ABS(a);
        b = ABS(b);
        if ( a == 0 ) return b;    // 0 is error value
        if ( b == 0 ) return a;
        while ( b > 0 )
        {   t = a % b;
            a = b;
            b = t;
        }
        return a;
}

Fraction::Fraction(int n, int d)   // d is not zero
{     int g;
      if ( d == 0 )
      {   std::cerr << " Fraction: Can't construct fraction"
                    << " with 0 denominator" << std::endl;
          abort();
      }
      if (n == 0) 
      {   num = 0; denom = 1; return;   }
      if (d < 0) 
      {    n = -n; d = -d;   }
      if ( (g = gcd(n,d)) != 1 )      // remove gcd
      {    n /= g; d /= g;   }
      num = n;
      denom = d;
}

bool Fraction::operator <(const Fraction& y) const
{    return( ( num * y.denom < denom * y.num ) ); }

bool Fraction::operator >(const Fraction& y) const
{    return( ( num * y.denom > denom * y.num ) ); }

bool Fraction::operator >=(const Fraction& y) const
{    return( ( num * y.denom >= denom * y.num ) ); }

bool Fraction::operator <=(const Fraction& y) const
{    return( ( num * y.denom <= denom * y.num ) ); }

int Fraction::floor() const
{   if ( num >= 0 ) return( num % denom );
    else return(-1 - (-num % denom));
}

int Fraction::ceiling() const
{   if ( num >= 0 ) return(1 + (num % denom));
    else return ( 1 - (-num % denom) );
}

Fraction Fraction::operator /(const Fraction& y) const
{    return Fraction( num * y.denom, denom * y.num ); }

Fraction Fraction::operator *(const Fraction& y) const
{    return Fraction( num * y.num, denom * y.denom ); }

Fraction Fraction::operator +(const Fraction& y) const
{    if ( num == 0 )
         return makeFraction(y.num, y.denom);
     else if ( y.num == 0 )
         return makeFraction(num, denom);
     else
         return Fraction(num * y.denom + y.num * denom,
                            denom * y.denom);
}

Fraction Fraction::operator -() const
{    return makeFraction(-num, denom);   }

Fraction Fraction::operator -(const Fraction& y) const
{    if ( num == 0 )           // trivial cases
          return  makeFraction(-y.num, y.denom);
     else if ( y.num == 0 )
          return makeFraction(num, denom);
     else                      // sub fractions
          return Fraction(num * y.denom - y.num * denom,
                            denom * y.denom);
}

ostream& operator <<(ostream& out, Fraction& x)
{   x.display(out);
    return out;
}

void Fraction::read(istream& in /* = cin */)
{   int num, denom;
    char c;
    in >> num >> c;
    if ( c == '/' )
    {    in >> denom;
         *this = Fraction(num, denom);
    }
    else    // clear() sets error state
         in.clear( ios::badbit | in.rdstate() );
}

istream& operator >>(istream& in, Fraction& f)
{    f.read(in);
     return in;
}
