///////    ArbList.h    ///////
#ifndef Arblist_SEEN__
#define Arblist_SEEN__
// linked list of arbitrary entries
#include <iostream>
typedef void *Any;

class ArbList
{ class ArbCell;
  public:
    typedef bool (* EQ_FN)(Any, Any);     // equality function type
    typedef void (* DISP_FN)(Any);        // display function type

    ArbList() : head(NULL) { }             // default constructor
    ArbList(Any c, EQ_FN eq, DISP_FN d);   // constructor  (1)
    static ArbCell* next(ArbCell* p);      // next cell
    static Any content(ArbCell* p);        // content of list item
    static bool isEnd(ArbCell* p)          // end test
    { return p==NULL; }          
    ArbCell* first() { return head; }      // first cell 
    ArbCell* last();                       // last cell
    ArbCell* find(Any c);                  // first item equals c
    bool substitute(Any r, Any s);         // r for first s on list
    int remove(Any c);                     // c from entire list
    void remove(ArbCell* cell);            // remove given cell
    int shorten(int n);                    // remove first n cells
    bool putOn(Any c);                     // insert in front
    bool insert(Any c, ArbCell* cell);     // insert after cell
    bool append(Any c)                     // insert at end
    { return insert(c, last()); }
    void display(ArbCell* p) const;        // display from p to end
    void display() const                   // display whole list
    { display(head); } 
    bool isEmpty() const { return head==NULL; }
    ~ArbList();                            // destructor
  protected:
    ArbCell* head;    // first cell of list
    void free();      // free all cells
    EQ_FN equal;      // supplied equality tester
    DISP_FN dispfn;   // supplied displayer
  public:
    class ArbCell
    { friend class ArbList;
      private:
        Any item;
        ArbCell* next;
        ArbCell(Any c = NULL, ArbCell* ptr = NULL)
        :  item(c), next(ptr) { }    // constructor
    };
};
#endif /*arblist_SEEN__ */
