///////     ArbTree.h    ///////
#ifndef ArbTree__SEEN__
#define ArbTree__SEEN__
#include "TreeNode.h"

// ArbTree is the generic tree class
// It also provides preorder, postorder, and inorder
//   iterators

class ArbTree
{ public:
    class Iterator;
    enum TRAVERSAL {PREORDER, INORDER, POSTORDER};
    static TreeNode* leaf(any item=NULL, InNode* parent=NULL)
    {  return new TreeNode(item, parent);  }
    static InNode* parent(any item=NULL)
    {  return new InNode(item);  }
    static int size(TreeNode* n);      // number of nodes from n
    static int height(TreeNode* n);    // height of tree from n

    explicit ArbTree(TreeNode* r=NULL)
    : root(r) { } // init tree with root node
    int size() const { return size(root); }
    int height() const { return height(root); }
    /*   ...  */
    ~ArbTree() {  free(root); }
    Iterator begin(TRAVERSAL ord=PREORDER) 
    { return Iterator(root, ord); }
    Iterator end(TRAVERSAL ord=PREORDER) 
    { return Iterator(NULL, ord); }
  protected:
    ArbTree& operator=(const ArbTree& t)
    { return *this; }  // prevents assignment

    ArbTree(const ArbTree& t)
    {  }  // prevents copy construction
  private:
    void free(TreeNode* n);
    TreeNode* root;              // root of tree
  public:
   class Iterator
   {  friend class ArbTree;
      public:
        Iterator& operator++();  // advance
        TreeNode& operator*()
        { return *current; }
        bool operator!=(const Iterator& b) const
        { return (current!=b.current); }
      private:
        TreeNode* postNext();
        TreeNode* inNext();
        TreeNode* preNext();
        Iterator(TreeNode* n, TRAVERSAL ord)
        : current(n), mode(ord) { }
        TreeNode* current;  // node being visited
        TRAVERSAL mode;     // traversal mode
   };
};
#endif
