///////     TreeNode.h    ///////
#ifndef TreeNode__SEEN__
#define TreeNode__SEEN__
typedef void* any;

//  TreeNode is a basic node, which can 
//  be used directly as a leaf node

class InNode;

class TreeNode
{   friend class ArbTree;
    friend class InNode;
  public:
    virtual bool isLeaf() const { return true; }
    void setParent(InNode* p) { parent = p; }
    InNode* getParent() { return parent; }
    virtual void setValue(any it) { value = it; }
    virtual void* getValue() { return value; }
  protected:
    any value;
    InNode* parent;
  private:
    explicit TreeNode(any it=0, InNode* p=0)   // constructor
    :  value(it), parent(p) { } 
};

class ChildCell
{   friend class ArbTree;
    friend class InNode;
  private:
    explicit ChildCell(TreeNode* c) 
    { child = c; next=NULL;}
    TreeNode* child;
    ChildCell* next;
};

//  InNode is an internal tree node
//  Children of this node are
//    kept in a list of ChildCells

class InNode : public TreeNode
{   friend class ArbTree;
  public:
    bool isLeaf() const { return false; }
    TreeNode* getChild(int i);  // 0-based indexing
    void addChild(TreeNode* n);
    bool deleteChild(int i);  
    bool replaceChild(int i, TreeNode*);  
    int getChildCount() { return childCount; }
    TreeNode* nextChild(TreeNode* ch); // next sibling to ch
  protected:
    ChildCell* children;   // linked list of children
    int childCount;
  private:
    explicit InNode(any v = 0);
    ChildCell* getCell(int i);
};
#endif
