///////    VectorND.C    ///////
#pragma implementation
#include <iostream>
#include <stdlib.h>
#include "VectorND.h"

template <typename T, int n>
VectorND<T,n>::VectorND(T v0)        // constructor
{    
     for ( int i=0 ; i < n ; i++ )
          vec[i] = v0;
}

template <typename T, int n>
VectorND<T,n>::VectorND (T v[n])      // init elements by array
{   
     for ( int i=0 ; i < n ; i++ )
          vec[i] = v[i];
}

template <typename T, int n>
VectorND<T,n> VectorND<T,n>::operator +(const VectorND<T,n> a) const
{    int i;
     VectorND<T,n> ans;
     for ( i=0 ; i < n ; i++ )
          ans.vec[i] = a.vec[i] + vec[i];
     return ans;
}

template <typename T, int n>
T VectorND<T,n>::inner(const VectorND<T,n>& v) const
{   T ans = vec[0]*v.vec[0];
    for ( int i=1; i < n; i++ )
        ans += vec[i]*v.vec[i];
    return ans;
}

template <typename T, int n>
T& VectorND<T,n>::operator [](int i)  // base 1 indexing
{    if ( i > 0 && i <= n )
          return vec[i-1];
     else
     {  cerr << "VectorND index out of range" 
             << endl;
        exit(1);
     }
}

template <typename T, int n>
const T& VectorND<T,n>::operator[](int i) const    // read-only access
{    if ( i > 0 && i <= n )
          return vec[i-1];
     else
     {  cerr << "VectorND index out of range" 
             << endl;
        exit(1);
     }
}

template <typename T, int n>
void VectorND<T,n>::display() const
{    const T *p = vec;
     int i = n;
     cout << '(';
     while ( i-- > 0 )
     {    cout << *p++;
          if ( i > 0 ) cout << "  ";
     }
     cout << ")";
}

//// unattached operator*
template <typename T, int n> 
VectorND<T,n>  operator*(int m, const VectorND<T,n>& v)
{    VectorND<T,n> ans;
     for (int i=0; i<n; i++)
         ans.vec[i] = v.vec[i]*m;
     return ans;
}

template <typename T, int n> 
VectorND<T,n>  operator*(const VectorND<T,n>& v, int m)
{    
     return m*v;
}

/////////////// Explicit Instantiations //////////
template VectorND<double,3>;

//template
//VectorND<double,3> operator* <double,3> 
//(const VectorND<double,3>& v, int m);

template
VectorND<double,3> operator* <double,3> 
(int m, VectorND<double,3> const & v);
