///////    HashTable.C    ///////
#pragma implementation
#include <iostream>
#include <string.h>
#include <limits.h>
#include "HashTable.h"

// file scope member function
static unsigned str_to_u(const char *str) 
{    unsigned value = 7u; 
     char c;
     while ( (c = *str) != '\0' )
     {    value *= c;
          str++;
     }
     return value;
}

template <typename T>
HashTable<T>::HashTable
(KEY_FN fn, unsigned k /* = 8 */, unsigned m /* = H_MULT */)
:   getkey(fn), bits(k), mult(m), len(0)
{    int size = 1 << bits;
     ht = new List<T>*[size];
     for ( int i = 0; i < size; i++ )
          ht[i] = NULL;
}

template <typename T>
HashTable<T>::~HashTable()
{    int size = 1 << bits;
     for ( int i = 0; i < size; i++ )
         if ( ht[i] ) delete(ht[i]);
     delete(ht);
}

template <typename T>
unsigned HashTable<T>::hash(const char* key) const
{    unsigned val = 0;
     val = str_to_u(key);
     return( (val*mult) >> (CHAR_BIT*sizeof(unsigned) - bits));
}

template <typename T>
bool HashTable<T>::get(const char* key, T& rval)
{    List<T>* a = ht[hash(key)];
     if ( !a ) return false;
     T val;
     List<T>::Iterator it = a->begin();
     for (; it != a->end(); ++it)
     {  val = *it;
        if ( strcmp (key, getkey(val)) == 0 )
        {    rval = val;
             return true;      // entry found 
        }
     }
     return false;             // not found
}

template <typename T>
bool HashTable<T>::isOn(char* key) const   // record already on table?
{    List<T>* a = ht[hash(key)];
     if ( !a || ! onList(*a, key) )
        return false;
     return true;
}

template <typename T>           // static private member function
bool HashTable<T>::onList
(List<T>& l, char* key) const
{    T val;
     List<T>::Iterator it = l.begin();
     for (; it != l.end(); ++it)
     {  val = *it;
        if ( strcmp (key, getkey(val)) == 0 )
             return true;    // entry found 
     }
     return false;           // not found
}

template <typename T>
bool HashTable<T>::rid(const char* key)
{    unsigned code = hash(key);
     List<T>* a = ht[code];
     if ( !a ) return false;
     T val;
     List<T>::Iterator it = a->begin();
     for (; it != a->end(); ++it)
     {  val = *it;
        if ( strcmp (key, getkey(val)) == 0 )
        {  a->remove(val);  len--;
           if ( a->isEmpty() )
               ht[code] = NULL;
           return true;
        }
     }
     return false;
}

// enter new entry into hash table
template <typename T>
void HashTable<T>::put(T& a)         // enter a into hash table
{    char *key = getkey(a);
     unsigned hc = hash(key);
     List<T>* l = ht[hc];
     if ( l == NULL )                // loccation not used before
     {   ht[hc] = new List<T>(a);
         len++; return;
     }
     if ( ! onList(*l, key) )       // if not already in table
     {  len++;  l->putOn(a); }      // put record on table
     return;
}

#include "Employee.h"
template class HashTable<Employee>;
