#ifndef HashTable_SEEN__
#define HashTable_SEEN__
#pragma interface
///////    HashTable.h    ///////

// Each table entry has a key in the form of a string.
// The key string is unique: two entries are regarded to
//     be the same if they have the same key
// A function getkey that extracts a key from an entry must
//     also be defined
// Usage:  HashTable<empolyee> ht(n)
//    gives a hash table of employees, size 2^n, [n=8]
//    getkey can return the SS No. for example
#include "List.h"

template <typename T>
class HashTable
{ public:
    typedef char* (* KEY_FN)(T&);       // client-supplied key function
    enum {H_MULT=2640025017u};          // default multiplier
    HashTable(KEY_FN fn,                // constructor
              unsigned k = 8,   
              unsigned m = H_MULT);
    void put(T& a);                     // enters record a
    bool get(const char *key, T& r);    // returns record in r
    bool rid(const char* key);          // removes a record
    bool isOn(char* key) const;         // is record is on table?
    unsigned length() const             // returns length of table 
    { return len; }
    ~HashTable();
  private:
    List<T>** ht;                     // table of linked lists
    KEY_FN getkey;                    // key function
    unsigned hash(const char*) const; // builtin hash function 
    bool onList(List<T>& l,           // is record on list l?
                char* key) const;
    unsigned len;                     // No. of entries on table
    unsigned mult;                    // hash multiplier
    unsigned bits;                    // hash table 2^bits
};
#endif
