///////    List.C    ///////
#pragma implementation
#include "List.h"

template <typename T>
List<T>::Cell*  List<T>::last() const
{   List<T>::Cell* p = head;
    while( p && p->next ) p =  p->next; 
    return p;
}

template <typename T>
bool List<T>::equal(const T& r, const T& s)
{   return(r == s);  }

template <typename T>
List<T>::~List() {   free();  }

template <typename T>
void List<T>::free() // private method
{  List<T>::Cell* n, *p = head;
   while (p)
   {   n = p->next;
       delete p;
       p=n;
   }
}

template <typename T>
int List<T>::shorten(int n)
{  while (head && n--)
   {   List<T>::Cell* tmp=head;
       head = head->next;
       delete(tmp);
   }
   return -n;
}

template <typename T>
int List<T>::remove(T& c)
{  List<T>::Cell *tmp, *p = head;
   int count = 0;
   while (p->next)      // treat all but head cell
   {  if ( equal((p->next)->getValue(), c) )
      {   count++;
          tmp = p->next;
          p->next = tmp->next;
          delete(tmp); // free up store
      }
      else
             p = p->next;
   }
   if ( equal(head->getValue(), c) ) // treat head cell
   {  tmp = head;
      head = head->next;
      delete(tmp);
      count++;
   }
   return count;        // number of items removed
}

template <typename T>
void List<T>::remove(List<T>::Cell* z)      // del list cell
{   List<T>::Cell* tmp;
    if ( head == z )                  // treat head cell
    {   tmp = head;
        head = head->next;
        delete(tmp);
        return;
    }
    List<T>::Cell* p = head;
    while (p->next)                   // treat other cells
        if ( p->next == z )
        {   tmp=p->next;
            p->next=(p->next)->next;
            delete(tmp);              // free up store
            return;
        }
        else p= p->next;
    return;                           // no entry is 0
}

template <typename T>
bool List<T>::putOn(T& c)
{   List<T>::Cell* tmp = new List<T>::Cell(c,head);
    if ( tmp )
    {  head = tmp;
       return true;
    }
    else return false; // failed
}

// insert after entry e
template <typename T>
bool List<T>::insert(T& c, List<T>::Cell* e)
{   List<T>::Cell* tmp = new List<T>::Cell(c,e->next);
    if ( tmp )
    {  if (e) e->next = tmp;
       else head = tmp;
       return true;
    }
    else return false; // failed
}

template <typename T>
List<T>::Cell* List<T>::find(T& c) const
{   for( List<T>::Cell* p = head; p ; p = p->next )
       if( equal(p->getValue(), c) )
           return p;
    return NULL;  // c not on list
}

template <typename T>
bool List<T>::substitute(T& r, T& s)
{   for( List<T>::Cell* p = head; p ; p = p->next )
       if( equal(p->getValue(), s) )
       {   p->setValue(r);
           return true;
       }
    return false;  // s not on list
}

// display from p to end
template <typename T>
void List<T>::display
   (List<T>::Cell* p, std::ostream& out) const
{   out << "(";
    while ( p )
    {    out << p->getValue();   // call overloaded <<
         if ( p = p->next ) out << ", ";
    }
    out << ")";
}

template <typename T>
List<T>::Iterator& 
List<T>::Iterator::operator++()
{  if ( cur != NULL )  
       cur = cur->next;
   return *this;
}

template <typename T>
const List<T>::Iterator& 
List<T>::Iterator::operator++() const
{  if ( cur != NULL )  
       (const_cast< Iterator* >(this))->cur = cur->next;
   return *this;
}

template<>      // specialization
bool List<char*>::equal(char* const & r, char* const & s)
{  return (! strcmp(r,s));  }

/////////////  explicit template instantiations   ////////////
////for use by testList.C

#include "Employee.h"
template class List<Employee>;

template 
std::ostream& operator<< < Employee >
    (std::ostream& out, const List<Employee>& list);

template 
std::ostream& operator<< < Employee >
    (std::ostream& out, const List<Employee>* list);

// explicit template instantiation --- for use with testListString.C
template class List<char*>;
