#ifndef TmpLIST__SEEN
#define TmpLIST__SEEN
///////    List.h    ///////
#pragma interface
#include <iostream>
// linked list template
// type T must define operator== and overload <<

template <typename T>
class List
{ class Cell;
  class Iterator;
  public:
    List() : head(NULL) { }          // empty list constructor 
    List(T& c)                       // list with first cell
      : head (new Cell(c)) { }
    Cell* last() const;              // last cell
    Cell* find(T& c) const;          // first value equals c
    bool substitute(T& r, T& s);     // r for first s on list
    int remove(T& c);                // c from entire list
    void remove(Cell* cell);         // remove given cell
    int shorten(int n);              // remove first n cells
    static T& content(Cell* p)
      {  return p->getValue();  }
    bool putOn(T& c);                // insert in front
    bool insert(T& c, Cell* cell);   // insert after cell
    int append(T& c)                 // insert at end
      { return insert(c, last()); }
    int isEmpty() const
      { return head==NULL; }
    void display(Cell* p,            // display from p to end
       std::ostream& out=std::cout) const;
    void display(std::ostream& out=std::cout) const
      { display(head, out); }        // display whole list
    ~List();                         // destructor

// nested slave class
    class Cell
    {   friend class List<T>;
        friend class List<T>::Iterator;
      public:
        T& getValue() { return val; }
        void setValue(T& v) { val = v; }
        T& val;                      // store pointer to type T
      private:
        Cell* next;
        Cell(T& c, Cell* ptr = NULL)
        :  val(c), next(ptr) { }    // constructor
    };
  private:
    static bool equal(const T&, const T&);  // equality test
    Cell* head;                             // first cell of list
    void free();                            // free all cells

// nested Iterator
  public:
    class Iterator
    {  friend class List<T>;
       public:
          Iterator& operator++();
          const Iterator& operator++() const;
          const T& operator*() const
          { return cur->getValue(); }
          T& operator*()
          { return cur->getValue(); }
          bool operator!=(const Iterator& b) const
          { return (cur!=b.cur); }
       private:
          Iterator (Cell* c) : cur(c) {}
          Cell* cur;
    };
    typedef Iterator iterator;
    typedef const Iterator const_iterator;
    iterator begin() { return Iterator(head); }
    const_iterator begin() const { return Iterator(head); }
    iterator end() { return Iterator(NULL); }
    const_iterator end() const { return Iterator(NULL); }
};

template <typename T>
std::ostream& operator<<
    (std::ostream& out, const List<T>& list)
{   list.display(out);  return out;  }

template <typename T>
std::ostream& operator<<
    (std::ostream& out, const List<T>* l)
{   l->display(out);  return out;  }
#endif
